/***
  @author Derek Callaway
  @version 1.0
  @return c-style string representation of random line
  @param file file to get a random line from

  Return a random line from a file excluding sh-style comments and blank lines.
*/

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<time.h>
#include<ctype.h>
#include<errno.h>
#include"randsrc.h"

/* Size of buffer for storing file entries. */
#define BUFSIZE 2048

typedef struct string_list {
	char*str;
	struct string_list*next;
}SLIST;

char*randsrc_getline(const char*const file)
{
	FILE*fi;
	register SLIST*current=NULL;
	register char*p,*p2;
	SLIST*start=NULL;
	char buf[BUFSIZE];
	unsigned int len,line,i,j;

        if(!(fi=fopen(file,"r")))
                randsrc_vexit("fopen");

	for(
			line=1,i=0;
			fgets(buf,sizeof(buf),fi);
			line++
        ){

		/* Skip comments */
		for(p2=buf;*p2&&isspace(*p2);p2++);
		if(*p2=='#')
			continue;

		if(len+1==BUFSIZE){
			fprintf(stderr,"WARNING: %s:%d Line too long!\n",file,line);
			continue;
		}

		len=strlen(buf);

		p=buf+len-1;
		do
			switch(*p)
			{
				case '\r':
				case '\n':
					break;
				default:
					p++;
					goto end;
			}
		while(--p!=buf);

end:
		/* Skip blank lines */
		if(p==buf)
			continue;

		*p=0;

		p=calloc(1,sizeof(*(current->next)));
		if(!p)
			randsrc_vexit("calloc");

		if(!start)
			start=current=(SLIST*)p;
		else
		{
			current->next=(SLIST*)p;
			current=current->next;
			i++;
		}	

                current->str=malloc(len+1);
		if(!current->str)
			randsrc_vexit("malloc");

                strcpy(current->str,buf);
        }

	if(!i)
	{
		errno=EIO;
		return NULL;
	}
	
        srand(time(NULL));
        j=(int)((float)(i+1)*rand()/(RAND_MAX));

	for(i=0,current=start;i<j;i++)
		current=current->next;

	return current->str;
}
